@@ -13,7 +13,7 @@ from . import __version__
 from .clients import BaseClient
 from .exceptions import TembaException, TembaSerializationException
 from .serialization import TembaObject, SimpleField, BooleanField, IntegerField, DatetimeField, ObjectField
-from .serialization import ObjectListField
+from .serialization import ObjectListField, ObjectDictField
 from .utils import format_iso8601, parse_iso8601
 
 
@@ -70,7 +70,9 @@ class UtilsTest(TembaTest):
 
     def test_format_iso8601(self):
         d = datetime.datetime(2014, 1, 2, 3, 4, 5, 6, UtilsTest.TestTZ())
-        self.assertEqual(format_iso8601(d), '2014-01-02T08:04:05.000006')
+        self.assertEqual(format_iso8601(d), '2014-01-02T08:04:05.000006Z')
+        # it should return None when no datetime given
+        self.assertIs(format_iso8601(None), None)
 
     def test_parse_iso8601(self):
         dt = datetime.datetime(2014, 1, 2, 3, 4, 5, 0, pytz.UTC)
@@ -92,6 +94,7 @@ class TestType(TembaObject):
     doh = DatetimeField()
     gem = ObjectField(item_class=TestSubType)
     hum = ObjectListField(item_class=TestSubType)
+    meh = ObjectDictField(item_class=TestSubType)
 
 
 class FieldsTest(TembaTest):
@@ -134,6 +137,21 @@ class FieldsTest(TembaTest):
         self.assertRaises(TembaSerializationException, field.deserialize, None)
         self.assertRaises(TembaSerializationException, field.deserialize, "")
 
+    def test_object_dict(self):
+        field = ObjectDictField(item_class=TestSubType)
+        self.assertEqual(field.serialize({'a': TestSubType.create(zed='c'), 'b': TestSubType.create(zed=2)}),
+                         {'a': {'zed': 'c'}, 'b': {'zed': 2}})
+
+        self.assertRaises(TembaSerializationException, field.serialize, "Not a list")
+
+        obj_dict = field.deserialize({'a': {'zed': 'c'}, 'b': {'zed': 2}})
+        self.assertEqual(len(obj_dict), 2)
+        self.assertEqual(obj_dict['a'].zed, 'c')
+        self.assertEqual(obj_dict['b'].zed, 2)
+
+        self.assertRaises(TembaSerializationException, field.deserialize, None)
+        self.assertRaises(TembaSerializationException, field.deserialize, "")
+
 
 class TembaObjectTest(TembaTest):
     def test_create(self):
@@ -152,13 +170,17 @@ class TembaObjectTest(TembaTest):
                                     'bar': 123,
                                     'doh': '2014-01-02T03:04:05',
                                     'gem': {'zed': 'c'},
-                                    'hum': [{'zed': 'b'}]})
+                                    'hum': [{'zed': 'b'}],
+                                    'meh': {'a': {'zed': 'c'}, 'b': {'zed': 'd'}}})
         self.assertEqual(obj.foo, 'a')
         self.assertEqual(obj.bar, 123)
         self.assertEqual(obj.doh, datetime.datetime(2014, 1, 2, 3, 4, 5, 0, pytz.UTC))
         self.assertEqual(obj.gem.zed, 'c')
         self.assertEqual(len(obj.hum), 1)
         self.assertEqual(obj.hum[0].zed, 'b')
+        self.assertEqual(len(obj.meh), 2)
+        self.assertEqual(obj.meh['a'].zed, 'c')
+        self.assertEqual(obj.meh['b'].zed, 'd')
 
         # exception when object list field receives non-list
         self.assertRaises(TembaSerializationException, TestType.deserialize,
@@ -167,14 +189,16 @@ class TembaObjectTest(TembaTest):
     def test_serialize(self):
         obj = TestType.create(foo='a', bar=123, doh=datetime.datetime(2014, 1, 2, 3, 4, 5, 0, pytz.UTC),
                               gem=TestSubType.create(zed='a'),
-                              hum=[TestSubType.create(zed='b')])
+                              hum=[TestSubType.create(zed='b')],
+                              meh={'a': TestSubType.create(zed='c'), 'b': TestSubType.create(zed='d')})
 
         json_obj = obj.serialize()
         self.assertEqual(json_obj, {'foo': 'a',
                                     'bar': 123,
-                                    'doh': '2014-01-02T03:04:05.000000',
+                                    'doh': '2014-01-02T03:04:05.000000Z',
                                     'gem': {'zed': 'a'},
-                                    'hum': [{'zed': 'b'}]})
+                                    'hum': [{'zed': 'b'}],
+                                    'meh': {'a': {'zed': 'c'}, 'b': {'zed': 'd'}}})
 
 
 class BaseClientTest(TembaTest):
